@@ -1,22 +1,25 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import errno
-from functools import partial
+import os
 from collections import Counter
+from functools import partial
+from qt.core import QDialog, QModelIndex, QObject, QTimer
 
-from PyQt5.Qt import QObject, QTimer, QModelIndex
-
-from calibre.constants import isosx
-from calibre.gui2 import error_dialog, question_dialog
-from calibre.gui2.dialogs.delete_matching_from_device import DeleteMatchingFromDeviceDialog
+from calibre.constants import ismacos
+from calibre.gui2 import Aborted, error_dialog, question_dialog
+from calibre.gui2.actions import InterfaceAction
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.confirm_delete_location import confirm_location
-from calibre.gui2.actions import InterfaceAction
+from calibre.gui2.dialogs.delete_matching_from_device import (
+    DeleteMatchingFromDeviceDialog
+)
 from calibre.utils.recycle_bin import can_recycle
 
 single_shot = partial(QTimer.singleShot, 10)
@@ -35,8 +38,10 @@ class MultiDeleter(QObject):  # {{{
                 _('You are trying to delete %d books. '
                     'Sending so many files to the Recycle'
                     ' Bin <b>can be slow</b>. Should calibre skip the'
-                    ' Recycle Bin? If you click Yes the files'
-                    ' will be <b>permanently deleted</b>.')%len(ids)):
+                    ' recycle bin? If you click Yes the files'
+                    ' will be <b>permanently deleted</b>.')%len(ids),
+                add_abort_button=True
+            ):
                 self.permanent = True
         self.gui = gui
         self.failures = []
@@ -73,8 +78,7 @@ class MultiDeleter(QObject):  # {{{
         self.pd = None
         self.model.db.commit()
         self.model.db.clean()
-        self.model.books_deleted()
-        self.gui.tags_view.recount()
+        self.model.books_deleted()  # calls recount on the tag browser
         self.callback(self.deleted_ids)
         if self.failures:
             msg = ['==> '+x[1]+'\n'+x[2] for x in self.failures]
@@ -87,7 +91,7 @@ class MultiDeleter(QObject):  # {{{
 class DeleteAction(InterfaceAction):
 
     name = 'Remove Books'
-    action_spec = (_('Remove books'), 'remove_books.png', _('Delete books'), 'Backspace' if isosx else 'Del')
+    action_spec = (_('Remove books'), 'remove_books.png', _('Delete books'), 'Backspace' if ismacos else 'Del')
     action_type = 'current'
     action_add_menu = True
     action_menu_clone_qaction = _('Remove selected books')
@@ -107,7 +111,7 @@ class DeleteAction(InterfaceAction):
     def drop_event(self, event, mime_data):
         mime = 'application/calibre+from_library'
         if mime_data.hasFormat(mime):
-            self.dropped_ids = tuple(map(int, str(mime_data.data(mime)).split()))
+            self.dropped_ids = tuple(map(int, mime_data.data(mime).data().split()))
             QTimer.singleShot(1, self.do_drop)
             return True
         return False
@@ -123,11 +127,12 @@ class DeleteAction(InterfaceAction):
         self.delete_menu = self.qaction.menu()
         m = partial(self.create_menu_action, self.delete_menu)
         m('delete-specific',
-                _('Remove files of a specific format from selected books...'),
+                _('Remove files of a specific format from selected books'),
                 triggered=self.delete_selected_formats)
         m('delete-except',
                 _('Remove all formats from selected books, except...'),
                 triggered=self.delete_all_but_selected_formats)
+        self.delete_menu.addSeparator()
         m('delete-all',
                 _('Remove all formats from selected books'),
                 triggered=self.delete_all_formats)
@@ -153,11 +158,11 @@ class DeleteAction(InterfaceAction):
         for x in ids:
             fmts_ = db.formats(x, index_is_id=True, verify_formats=False)
             if fmts_:
-                for x in frozenset([x.lower() for x in fmts_.split(',')]):
+                for x in frozenset(x.lower() for x in fmts_.split(',')):
                     c[x] += 1
         d = SelectFormats(c, msg, parent=self.gui, exclude=exclude,
                 single=single)
-        if d.exec_() != d.Accepted:
+        if d.exec_() != QDialog.DialogCode.Accepted:
             return None
         return d.selected_formats
 
@@ -166,7 +171,7 @@ class DeleteAction(InterfaceAction):
         if not rows or len(rows) == 0:
             d = error_dialog(self.gui, err_title, _('No book selected'))
             d.exec_()
-            return set([])
+            return set()
         return set(map(self.gui.library_view.model().id, rows))
 
     def remove_format_by_id(self, book_id, fmt):
@@ -182,14 +187,14 @@ class DeleteAction(InterfaceAction):
         self.gui.library_view.model().refresh_ids([book_id])
         self.gui.library_view.model().current_changed(self.gui.library_view.currentIndex(),
                 self.gui.library_view.currentIndex())
-        self.gui.tags_view.recount()
+        self.gui.tags_view.recount_with_position_based_index()
 
     def restore_format(self, book_id, original_fmt):
         self.gui.current_db.restore_original_format(book_id, original_fmt)
         self.gui.library_view.model().refresh_ids([book_id])
         self.gui.library_view.model().current_changed(self.gui.library_view.currentIndex(),
                 self.gui.library_view.currentIndex())
-        self.gui.tags_view.recount()
+        self.gui.tags_view.recount_with_position_based_index()
 
     def delete_selected_formats(self, *args):
         ids = self._get_selected_ids()
@@ -205,7 +210,7 @@ class DeleteAction(InterfaceAction):
         m.current_changed(self.gui.library_view.currentIndex(),
                 self.gui.library_view.currentIndex())
         if ids:
-            self.gui.tags_view.recount()
+            self.gui.tags_view.recount_with_position_based_index()
 
     def delete_all_but_selected_formats(self, *args):
         ids = self._get_selected_ids()
@@ -223,7 +228,7 @@ class DeleteAction(InterfaceAction):
             bfmts = m.db.formats(id, index_is_id=True)
             if bfmts is None:
                 continue
-            bfmts = set([x.lower() for x in bfmts.split(',')])
+            bfmts = {x.lower() for x in bfmts.split(',')}
             rfmts = bfmts - set(fmts)
             if bfmts - rfmts:
                 # Do not delete if it will leave the book with no
@@ -235,7 +240,7 @@ class DeleteAction(InterfaceAction):
             m.current_changed(self.gui.library_view.currentIndex(),
                     self.gui.library_view.currentIndex())
             if ids:
-                self.gui.tags_view.recount()
+                self.gui.tags_view.recount_with_position_based_index()
 
     def delete_all_formats(self, *args):
         ids = self._get_selected_ids()
@@ -258,7 +263,7 @@ class DeleteAction(InterfaceAction):
             self.gui.library_view.model().current_changed(self.gui.library_view.currentIndex(),
                     self.gui.library_view.currentIndex())
             if ids:
-                self.gui.tags_view.recount()
+                self.gui.tags_view.recount_with_position_based_index()
 
     def remove_matching_books_from_device(self, *args):
         if not self.gui.device_manager.is_device_present:
@@ -274,8 +279,8 @@ class DeleteAction(InterfaceAction):
         to_delete = {}
         some_to_delete = False
         for model,name in ((self.gui.memory_view.model(), _('Main memory')),
-                           (self.gui.card_a_view.model(), _('Storage Card A')),
-                           (self.gui.card_b_view.model(), _('Storage Card B'))):
+                           (self.gui.card_a_view.model(), _('Storage card A')),
+                           (self.gui.card_b_view.model(), _('Storage card B'))):
             to_delete[name] = (model, model.paths_for_db_ids(ids))
             if len(to_delete[name][1]) > 0:
                 some_to_delete = True
@@ -376,19 +381,22 @@ class DeleteAction(InterfaceAction):
         if len(to_delete_ids) < 5:
             try:
                 view.model().delete_books_by_id(to_delete_ids)
-            except IOError as err:
+            except OSError as err:
                 if err.errno == errno.EACCES:
                     import traceback
-                    fname = getattr(err, 'filename', 'file') or 'file'
+                    fname = os.path.basename(getattr(err, 'filename', 'file') or 'file')
                     return error_dialog(self.gui, _('Permission denied'),
                             _('Could not access %s. Is it being used by another'
                             ' program? Click "Show details" for more information.')%fname, det_msg=traceback.format_exc(),
                             show=True)
-
+                raise
             self.library_ids_deleted2(to_delete_ids, next_id=next_id)
         else:
-            self.__md = MultiDeleter(self.gui, to_delete_ids,
-                    partial(self.library_ids_deleted2, next_id=next_id))
+            try:
+                self.__md = MultiDeleter(self.gui, to_delete_ids,
+                        partial(self.library_ids_deleted2, next_id=next_id))
+            except Aborted:
+                pass
 
     def delete_books(self, *args):
         '''
