@@ -13,7 +13,7 @@ from qt.core import Qt, QDialog
 from calibre.gui2.convert.look_and_feel_ui import Ui_Form
 from calibre.gui2.convert import Widget
 from calibre.ebooks.conversion.config import OPTIONS
-from polyglot.builtins import iteritems, unicode_type
+from polyglot.builtins import iteritems
 
 
 class LookAndFeelWidget(Widget, Ui_Form):
@@ -55,7 +55,7 @@ class LookAndFeelWidget(Widget, Ui_Form):
 
     def get_value_handler(self, g):
         if g is self.opt_change_justification:
-            ans = unicode_type(g.itemData(g.currentIndex()) or '')
+            ans = str(g.itemData(g.currentIndex()) or '')
             return ans
         if g is self.opt_filter_css:
             ans = set()
@@ -64,10 +64,10 @@ class LookAndFeelWidget(Widget, Ui_Form):
                 if w.isChecked():
                     ans = ans.union(item)
             ans = ans.union({x.strip().lower() for x in
-                unicode_type(self.filter_css_others.text()).split(',')})
+                str(self.filter_css_others.text()).split(',')})
             return ','.join(ans) if ans else None
         if g is self.opt_font_size_mapping:
-            val = unicode_type(g.text()).strip()
+            val = str(g.text()).strip()
             val = [x.strip() for x in val.split(',' if ',' in val else ' ') if x.strip()]
             return ', '.join(val) or None
         if g is self.opt_transform_css_rules:
@@ -77,7 +77,7 @@ class LookAndFeelWidget(Widget, Ui_Form):
     def set_value_handler(self, g, val):
         if g is self.opt_change_justification:
             for i in range(g.count()):
-                c = unicode_type(g.itemData(i) or '')
+                c = str(g.itemData(i) or '')
                 if val == c:
                     g.setCurrentIndex(i)
                     break
@@ -85,7 +85,7 @@ class LookAndFeelWidget(Widget, Ui_Form):
         if g is self.opt_filter_css:
             if not val:
                 val = ''
-            items = frozenset([x.strip().lower() for x in val.split(',')])
+            items = frozenset(x.strip().lower() for x in val.split(','))
             for key, vals in iteritems(self.FILTER_CSS):
                 w = getattr(self, 'filter_css_%s'%key)
                 if not vals - items:
@@ -114,7 +114,7 @@ class LookAndFeelWidget(Widget, Ui_Form):
     def font_key_wizard(self):
         from calibre.gui2.convert.font_key import FontKeyChooser
         d = FontKeyChooser(self, self.opt_base_font_size.value(),
-                unicode_type(self.opt_font_size_mapping.text()).strip())
+                str(self.opt_font_size_mapping.text()).strip())
         if d.exec_() == QDialog.DialogCode.Accepted:
             self.opt_font_size_mapping.setText(', '.join(['%.1f'%x for x in
                 d.fsizes]))
