@@ -50,7 +50,17 @@ from calibre.utils.date import (
 )
 from calibre.utils.filenames import make_long_path_useable
 from calibre.utils.icu import sort_key, strcmp
-from polyglot.builtins import iteritems, range, unicode_type
+from polyglot.builtins import iteritems
+
+
+def show_locked_file_error(parent, err):
+    import traceback
+    fname = getattr(err, 'filename', None)
+    p = 'Locked file: %s\n\n'%fname if fname else ''
+    error_dialog(parent, _('Permission denied'),
+            _('Could not change the on disk location of this'
+                ' book. Is it open in another program?'),
+            det_msg=p+traceback.format_exc(), show=True)
 
 
 def save_dialog(parent, title, msg, det_msg=''):
@@ -67,7 +77,7 @@ def clean_text(x):
 
 '''
 The interface common to all widgets used to set basic metadata
-class BasicMetadataWidget(object):
+class BasicMetadataWidget:
 
     LABEL = "label text"
 
@@ -87,7 +97,7 @@ class BasicMetadataWidget(object):
 '''
 
 
-class ToMetadataMixin(object):
+class ToMetadataMixin:
 
     FIELD_NAME = None
     allow_undo = False
@@ -231,7 +241,7 @@ class TitleEdit(EnLineEdit, ToMetadataMixin):
 
     @property
     def current_val(self):
-        title = clean_text(unicode_type(self.text()))
+        title = clean_text(str(self.text()))
         if not title:
             title = self.get_default()
         return title.strip()
@@ -341,6 +351,7 @@ class AuthorsEdit(EditWithComplete, ToMetadataMixin):
         self.dialog = parent
         self.books_to_refresh = set()
         EditWithComplete.__init__(self, parent)
+        self.set_clear_button_enabled(False)
         self.setToolTip(self.TOOLTIP)
         self.setWhatsThis(self.TOOLTIP)
         self.setEditable(True)
@@ -373,7 +384,11 @@ class AuthorsEdit(EditWithComplete, ToMetadataMixin):
             if d == QMessageBox.StandardButton.Cancel:
                 return
             if d == QMessageBox.StandardButton.Yes:
-                self.commit(self.db, self.id_)
+                try:
+                    self.commit(self.db, self.id_)
+                except PermissionError as err:
+                    show_locked_file_error(self, err)
+                    return
                 self.db.commit()
                 self.original_val = self.current_val
             else:
@@ -419,7 +434,7 @@ class AuthorsEdit(EditWithComplete, ToMetadataMixin):
     @property
     def current_val(self):
 
-        au = clean_text(unicode_type(self.text()))
+        au = clean_text(str(self.text()))
         if not au:
             au = self.get_default()
         return string_to_authors(au)
@@ -486,7 +501,7 @@ class AuthorSortEdit(EnLineEdit, ToMetadataMixin):
     @property
     def current_val(self):
 
-        return clean_text(unicode_type(self.text()))
+        return clean_text(str(self.text()))
 
     @current_val.setter
     def current_val(self, val):
@@ -507,7 +522,7 @@ class AuthorSortEdit(EnLineEdit, ToMetadataMixin):
         return self.db.new_api.author_sort_from_authors(authors, key_func=lambda x: x)
 
     def update_state(self, *args):
-        au = unicode_type(self.authors_edit.text())
+        au = str(self.authors_edit.text())
         au = re.sub(r'\s+et al\.$', '', au)
         au = self.author_sort_from_authors(string_to_authors(au))
 
@@ -533,13 +548,13 @@ class AuthorSortEdit(EnLineEdit, ToMetadataMixin):
             self.authors_edit.set_value(ans)
 
     def auto_generate(self, *args):
-        au = unicode_type(self.authors_edit.text())
+        au = str(self.authors_edit.text())
         au = re.sub(r'\s+et al\.$', '', au).strip()
         authors = string_to_authors(au)
         self.set_value(self.author_sort_from_authors(authors))
 
     def author_to_sort(self, *args):
-        au = unicode_type(self.authors_edit.text())
+        au = str(self.authors_edit.text())
         au = re.sub(r'\s+et al\.$', '', au).strip()
         if au:
             self.set_value(au)
@@ -598,6 +613,7 @@ class SeriesEdit(EditWithComplete, ToMetadataMixin):
 
     def __init__(self, parent):
         EditWithComplete.__init__(self, parent)
+        self.set_clear_button_enabled(False)
         self.set_separator(None)
         self.dialog = parent
         self.setSizeAdjustPolicy(
@@ -611,7 +627,7 @@ class SeriesEdit(EditWithComplete, ToMetadataMixin):
     @property
     def current_val(self):
 
-        return clean_text(unicode_type(self.currentText()))
+        return clean_text(str(self.currentText()))
 
     @current_val.setter
     def current_val(self, val):
@@ -643,7 +659,7 @@ class SeriesIndexEdit(make_undoable(QDoubleSpinBox), ToMetadataMixin):
     data_changed = pyqtSignal()
 
     def __init__(self, parent, series_edit):
-        super(SeriesIndexEdit, self).__init__(parent)
+        super().__init__(parent)
         self.valueChanged.connect(self.data_changed)
         self.dialog = parent
         self.db = self.original_series_name = None
@@ -1179,10 +1195,10 @@ class Cover(ImageView):  # {{{
             try:
                 with open(_file, "rb") as f:
                     cover = f.read()
-            except IOError as e:
+            except OSError as e:
                 d = error_dialog(
                         self, _('Error reading file'),
-                        _("<p>There was an error reading from file: <br /><b>") + _file + "</b></p><br />"+unicode_type(e))
+                        _("<p>There was an error reading from file: <br /><b>") + _file + "</b></p><br />"+str(e))
                 d.exec_()
             if cover:
                 orig = self.current_val
@@ -1340,7 +1356,7 @@ class RatingEdit(RatingEditor, ToMetadataMixin):  # {{{
     data_changed = pyqtSignal()
 
     def __init__(self, parent):
-        super(RatingEdit, self).__init__(parent)
+        super().__init__(parent)
         self.setToolTip(self.TOOLTIP)
         self.setWhatsThis(self.TOOLTIP)
         self.currentTextChanged.connect(self.data_changed)
@@ -1359,7 +1375,8 @@ class RatingEdit(RatingEditor, ToMetadataMixin):  # {{{
         self.original_val = self.current_val
 
     def commit(self, db, id_):
-        db.set_rating(id_, self.current_val, notify=False, commit=False)
+        if self.current_val != self.original_val:
+            db.set_rating(id_, self.current_val, notify=False, commit=False)
         return True
 
     def zero(self):
@@ -1379,6 +1396,7 @@ class TagsEdit(EditWithComplete, ToMetadataMixin):  # {{{
 
     def __init__(self, parent):
         EditWithComplete.__init__(self, parent)
+        self.set_clear_button_enabled(False)
         self.set_elide_mode(Qt.TextElideMode.ElideMiddle)
         self.currentTextChanged.connect(self.data_changed)
         self.lineEdit().setMaxLength(655360)  # see https://bugs.launchpad.net/bugs/1630944
@@ -1388,7 +1406,7 @@ class TagsEdit(EditWithComplete, ToMetadataMixin):  # {{{
 
     @property
     def current_val(self):
-        return [clean_text(x) for x in unicode_type(self.text()).split(',')]
+        return [clean_text(x) for x in str(self.text()).split(',')]
 
     @current_val.setter
     def current_val(self, val):
@@ -1404,12 +1422,15 @@ class TagsEdit(EditWithComplete, ToMetadataMixin):  # {{{
         self.current_val = tags
         self.update_items_cache(db.new_api.all_field_names('tags'))
         self.original_val = self.current_val
+        self.db = db
 
     @property
     def changed(self):
         return self.current_val != self.original_val
 
     def edit(self, db, id_):
+        ctrl_or_shift_pressed = (QApplication.keyboardModifiers() &
+                (Qt.KeyboardModifier.ControlModifier + Qt.KeyboardModifier.ShiftModifier))
         if self.changed:
             d = save_dialog(self, _('Tags changed'),
                     _('You have changed the tags. In order to use the tags'
@@ -1423,15 +1444,22 @@ class TagsEdit(EditWithComplete, ToMetadataMixin):  # {{{
                 self.original_val = self.current_val
             else:
                 self.current_val = self.original_val
-        d = TagEditor(self, db, id_)
-        if d.exec_() == QDialog.DialogCode.Accepted:
-            self.current_val = d.tags
-            self.update_items_cache(db.new_api.all_field_names('tags'))
+        if ctrl_or_shift_pressed:
+            from calibre.gui2.ui import get_gui
+            get_gui().do_tags_list_edit(None, 'tags')
+            self.update_items_cache(self.db.new_api.all_field_names('tags'))
+            self.initialize(self.db, id_)
+        else:
+            d = TagEditor(self, db, id_)
+            if d.exec_() == QDialog.DialogCode.Accepted:
+                self.current_val = d.tags
+                self.update_items_cache(db.new_api.all_field_names('tags'))
 
     def commit(self, db, id_):
-        self.books_to_refresh |= db.set_tags(
-                id_, self.current_val, notify=False, commit=False,
-                allow_case_change=True)
+        if self.changed:
+            self.books_to_refresh |= db.set_tags(
+                    id_, self.current_val, notify=False, commit=False,
+                    allow_case_change=True)
         return True
 
     def keyPressEvent(self, ev):
@@ -1453,6 +1481,7 @@ class LanguagesEdit(LE, ToMetadataMixin):  # {{{
 
     def __init__(self, *args, **kwargs):
         LE.__init__(self, *args, **kwargs)
+        self.set_clear_button_enabled(False)
         self.textChanged.connect(self.data_changed)
         self.setToolTip(self.TOOLTIP)
 
@@ -1565,7 +1594,7 @@ class IdentifiersEdit(QLineEdit, ToMetadataMixin):
 
     @property
     def current_val(self):
-        raw = unicode_type(self.text()).strip()
+        raw = str(self.text()).strip()
         parts = [clean_text(x) for x in raw.split(',')]
         ans = {}
         for x in parts:
@@ -1629,6 +1658,9 @@ class IdentifiersEdit(QLineEdit, ToMetadataMixin):
         identifier_found = self.parse_clipboard_for_identifier()
         if identifier_found:
             return
+        text = str(QApplication.clipboard().text()).strip()
+        if text.startswith('http://') or text.startswith('https://'):
+            return self.paste_prefix('url')
         try:
             prefix = gprefs['paste_isbn_prefixes'][0]
         except IndexError:
@@ -1639,14 +1671,14 @@ class IdentifiersEdit(QLineEdit, ToMetadataMixin):
         if prefix == 'isbn':
             self.paste_isbn()
         else:
-            text = unicode_type(QApplication.clipboard().text()).strip()
+            text = str(QApplication.clipboard().text()).strip()
             if text:
                 vals = self.current_val
                 vals[prefix] = text
                 self.current_val = vals
 
     def paste_isbn(self):
-        text = unicode_type(QApplication.clipboard().text()).strip()
+        text = str(QApplication.clipboard().text()).strip()
         if not text or not check_isbn(text):
             d = ISBNDialog(self, text)
             if not d.exec_():
@@ -1666,7 +1698,7 @@ class IdentifiersEdit(QLineEdit, ToMetadataMixin):
     def parse_clipboard_for_identifier(self):
         from calibre.ebooks.metadata.sources.prefs import msprefs
         from calibre.utils.formatter import EvalFormatter
-        text = unicode_type(QApplication.clipboard().text()).strip()
+        text = str(QApplication.clipboard().text()).strip()
         if not text:
             return False
 
@@ -1736,7 +1768,7 @@ class ISBNDialog(QDialog):  # {{{
         self.resize(sz)
 
     def accept(self):
-        isbn = unicode_type(self.line_edit.text())
+        isbn = str(self.line_edit.text())
         if not check_isbn(isbn):
             return error_dialog(self, _('Invalid ISBN'),
                     _('The ISBN you entered is not valid. Try again.'),
@@ -1744,7 +1776,7 @@ class ISBNDialog(QDialog):  # {{{
         QDialog.accept(self)
 
     def checkText(self, txt):
-        isbn = unicode_type(txt)
+        isbn = str(txt)
         if not isbn:
             sheet = ''
             extra = ''
@@ -1758,7 +1790,7 @@ class ISBNDialog(QDialog):  # {{{
         self.line_edit.setStyleSheet(sheet)
 
     def text(self):
-        return check_isbn(unicode_type(self.line_edit.text()))
+        return check_isbn(str(self.line_edit.text()))
 
 # }}}
 
@@ -1770,6 +1802,7 @@ class PublisherEdit(EditWithComplete, ToMetadataMixin):  # {{{
 
     def __init__(self, parent):
         EditWithComplete.__init__(self, parent)
+        self.set_clear_button_enabled(False)
         self.currentTextChanged.connect(self.data_changed)
         self.set_separator(None)
         self.setSizeAdjustPolicy(
@@ -1783,7 +1816,7 @@ class PublisherEdit(EditWithComplete, ToMetadataMixin):  # {{{
     @property
     def current_val(self):
 
-        return clean_text(unicode_type(self.currentText()))
+        return clean_text(str(self.currentText()))
 
     @current_val.setter
     def current_val(self, val):
@@ -1817,7 +1850,7 @@ class DateEdit(make_undoable(DateTimeEdit), ToMetadataMixin):
     data_changed = pyqtSignal()
 
     def __init__(self, parent, create_clear_button=True):
-        super(DateEdit, self).__init__(parent)
+        super().__init__(parent)
         self.setToolTip(self.TOOLTIP)
         self.setWhatsThis(self.TOOLTIP)
         self.dateTimeChanged.connect(self.data_changed)
@@ -1869,7 +1902,7 @@ class DateEdit(make_undoable(DateTimeEdit), ToMetadataMixin):
         elif ev.key() == Qt.Key.Key_Tab and is_date_undefined(self.current_val):
             ev.ignore()
         else:
-            return super(DateEdit, self).keyPressEvent(ev)
+            return super().keyPressEvent(ev)
 
 
 class PubdateEdit(DateEdit):
