@@ -1,12 +1,11 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-from PyQt5.Qt import QTextCharFormat
+from qt.core import QTextCharFormat, QTextFormat
 
 from calibre.ebooks.oeb.base import OEB_DOCS, OEB_STYLES
 from calibre.ebooks.oeb.polish.container import guess_type
@@ -25,7 +24,7 @@ def syntax_from_mime(name, mime):
     if mime.startswith('text/'):
         return 'text'
     if mime.startswith('image/') and mime.partition('/')[-1].lower() in {
-        'jpeg', 'jpg', 'gif', 'png'}:
+        'jpeg', 'jpg', 'gif', 'png', 'webp'}:
         return 'raster_image'
     if mime.endswith('+xml'):
         return 'xml'
@@ -43,7 +42,7 @@ def editor_from_syntax(syntax, parent=None):
         return Editor(syntax, parent=parent)
 
 
-SYNTAX_PROPERTY = QTextCharFormat.UserProperty
+SYNTAX_PROPERTY = QTextFormat.Property.UserProperty
 SPELL_PROPERTY = SYNTAX_PROPERTY + 1
 SPELL_LOCALE_PROPERTY = SPELL_PROPERTY + 1
 LINK_PROPERTY = SPELL_LOCALE_PROPERTY + 1
@@ -58,7 +57,7 @@ def syntax_text_char_format(*args):
     return ans
 
 
-class StoreLocale(object):
+class StoreLocale:
 
     __slots__ = ('enabled',)
 
