@@ -8,16 +8,16 @@ __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 import re
 from threading import Thread
 
-from PyQt5.Qt import (
+from qt.core import (
     QTextBrowser, QVBoxLayout, QDialog, QDialogButtonBox, QIcon, QLabel,
     QCheckBox, Qt, QListWidgetItem, QHBoxLayout, QListWidget, QPixmap,
-    QSpinBox, QStyledItemDelegate, QSize, QStyle, QPen,
-    QProgressBar, pyqtSignal, QApplication
+    QSpinBox, QStyledItemDelegate, QSize, QStyle, QPen, QPalette,
+    QProgressBar, pyqtSignal, QApplication, QAbstractItemView
 )
 
 from calibre import human_readable, fit_image, force_unicode
 from calibre.ebooks.oeb.polish.main import CUSTOMIZATION
-from calibre.gui2 import empty_index
+from calibre.gui2 import empty_index, question_dialog
 from calibre.gui2.tweak_book import tprefs, current_container, set_current_container
 from calibre.gui2.tweak_book.widgets import Dialog
 from calibre.utils.icu import numeric_sort_key
@@ -47,30 +47,38 @@ def customize_remove_unused_css(name, parent, ans):
     l.addWidget(c)
     d.la2 = label('<span style="font-size:small; font-style: italic">' + _(
         'Remove all class attributes from the HTML that do not match any existing CSS rules'))
-    d.m = m = QCheckBox(_('Merge CSS rules with identical selectors'))
+    d.m = m = QCheckBox(_('Merge CSS rules with identical &selectors'))
     m.setChecked(tprefs['merge_identical_selectors'])
     l.addWidget(m)
     d.la3 = label('<span style="font-size:small; font-style: italic">' + _(
         'Merge CSS rules in the same stylesheet that have identical selectors.'
     ' Note that in rare cases merging can result in a change to the effective styling'
     ' of the book, so use with care.'))
-    d.p = p = QCheckBox(_('Merge CSS rules with identical properties'))
+    d.p = p = QCheckBox(_('Merge CSS rules with identical &properties'))
     p.setChecked(tprefs['merge_rules_with_identical_properties'])
     l.addWidget(p)
     d.la4 = label('<span style="font-size:small; font-style: italic">' + _(
         'Merge CSS rules in the same stylesheet that have identical properties.'
     ' Note that in rare cases merging can result in a change to the effective styling'
     ' of the book, so use with care.'))
+    d.u = u = QCheckBox(_('Remove &unreferenced style sheets'))
+    u.setChecked(tprefs['remove_unreferenced_sheets'])
+    l.addWidget(u)
+    d.la5 = label('<span style="font-size:small; font-style: italic">' + _(
+        'Remove stylesheets that are not referenced by any content.'
+    ))
 
-    d.bb = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+    d.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
     d.l.addWidget(d.bb)
     d.bb.rejected.connect(d.reject)
     d.bb.accepted.connect(d.accept)
-    if d.exec_() != d.Accepted:
-        raise Abort()
+    ret = d.exec_()
     ans['remove_unused_classes'] = tprefs['remove_unused_classes'] = c.isChecked()
     ans['merge_identical_selectors'] = tprefs['merge_identical_selectors'] = m.isChecked()
     ans['merge_rules_with_identical_properties'] = tprefs['merge_rules_with_identical_properties'] = p.isChecked()
+    ans['remove_unreferenced_sheets'] = tprefs['remove_unreferenced_sheets'] = u.isChecked()
+    if ret != QDialog.DialogCode.Accepted:
+        raise Abort()
 
 
 def get_customization(action, name, parent):
@@ -78,6 +86,18 @@ def get_customization(action, name, parent):
     try:
         if action == 'remove_unused_css':
             customize_remove_unused_css(name, parent, ans)
+        elif action == 'upgrade_book':
+            ans['remove_ncx'] = tprefs['remove_ncx'] = question_dialog(
+                parent, _('Remove NCX ToC file'),
+                _('Remove the legacy Table of Contents in NCX form?'),
+                _('This form of Table of Contents is superseded by the new HTML based Table of Contents.'
+                  ' Leaving it behind is useful only if you expect this book to be read on very'
+                  ' old devices that lack proper support for EPUB 3'),
+                skip_dialog_name='edit-book-remove-ncx',
+                skip_dialog_msg=_('Ask this question again in the future'),
+                skip_dialog_skipped_value=tprefs['remove_ncx'],
+                yes_text=_('Remove NCX'), no_text=_('Keep NCX')
+            )
     except Abort:
         return None
     return ans
@@ -98,13 +118,13 @@ def show_report(changed, title, report, parent, show_current_diff):
     d.e = QTextBrowser(d)
     d.l.addWidget(d.e)
     d.e.setHtml(report)
-    d.bb = QDialogButtonBox(QDialogButtonBox.Close)
+    d.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Close)
     d.show_changes = False
     if changed:
-        b = d.b = d.bb.addButton(_('See what &changed'), d.bb.AcceptRole)
+        b = d.b = d.bb.addButton(_('See what &changed'), QDialogButtonBox.ButtonRole.AcceptRole)
         b.setIcon(QIcon(I('diff.png'))), b.setAutoDefault(False)
         connect_lambda(b.clicked, d, lambda d: setattr(d, 'show_changes', True))
-    b = d.bb.addButton(_('&Copy to clipboard'), d.bb.ActionRole)
+    b = d.bb.addButton(_('&Copy to clipboard'), QDialogButtonBox.ButtonRole.ActionRole)
     b.setIcon(QIcon(I('edit-copy.png'))), b.setAutoDefault(False)
 
     def copy_report():
@@ -114,7 +134,7 @@ def show_report(changed, title, report, parent, show_current_diff):
         cp.setText(text)
 
     b.clicked.connect(copy_report)
-    d.bb.button(d.bb.Close).setDefault(True)
+    d.bb.button(QDialogButtonBox.StandardButton.Close).setDefault(True)
     d.l.addWidget(d.bb)
     d.bb.rejected.connect(d.reject)
     d.bb.accepted.connect(d.accept)
@@ -133,17 +153,17 @@ class ImageItemDelegate(QStyledItemDelegate):
         return QSize(300, 100)
 
     def paint(self, painter, option, index):
-        name = index.data(Qt.DisplayRole)
-        sz = human_readable(index.data(Qt.UserRole))
-        pmap = index.data(Qt.UserRole+1)
+        name = index.data(Qt.ItemDataRole.DisplayRole)
+        sz = human_readable(index.data(Qt.ItemDataRole.UserRole))
+        pmap = index.data(Qt.ItemDataRole.UserRole+1)
         irect = option.rect.adjusted(0, 5, 0, -5)
         irect.setRight(irect.left() + 70)
         if pmap is None:
             pmap = QPixmap(current_container().get_file_path_for_processing(name))
             scaled, nwidth, nheight = fit_image(pmap.width(), pmap.height(), irect.width(), irect.height())
             if scaled:
-                pmap = pmap.scaled(nwidth, nheight, transformMode=Qt.SmoothTransformation)
-            index.model().setData(index, pmap, Qt.UserRole+1)
+                pmap = pmap.scaled(nwidth, nheight, transformMode=Qt.TransformationMode.SmoothTransformation)
+            index.model().setData(index, pmap, Qt.ItemDataRole.UserRole+1)
         x, y = (irect.width() - pmap.width())//2, (irect.height() - pmap.height())//2
         r = irect.adjusted(x, y, -x, -y)
         QStyledItemDelegate.paint(self, painter, option, empty_index)
@@ -151,9 +171,9 @@ class ImageItemDelegate(QStyledItemDelegate):
         trect = irect.adjusted(irect.width() + 10, 0, 0, 0)
         trect.setRight(option.rect.right())
         painter.save()
-        if option.state & QStyle.State_Selected:
-            painter.setPen(QPen(option.palette.color(option.palette.HighlightedText)))
-        painter.drawText(trect, Qt.AlignVCenter | Qt.AlignLeft, name + '\n' + sz)
+        if option.state & QStyle.StateFlag.State_Selected:
+            painter.setPen(QPen(option.palette.color(QPalette.ColorRole.HighlightedText)))
+        painter.drawText(trect, Qt.AlignmentFlag.AlignVCenter | Qt.AlignmentFlag.AlignLeft, name + '\n' + sz)
         painter.restore()
 
 
@@ -173,8 +193,8 @@ class CompressImages(Dialog):
         c = current_container()
         for name in sorted(get_compressible_images(c), key=numeric_sort_key):
             x = QListWidgetItem(name, i)
-            x.setData(Qt.UserRole, c.filesize(name))
-        i.setSelectionMode(i.ExtendedSelection)
+            x.setData(Qt.ItemDataRole.UserRole, c.filesize(name))
+        i.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
         i.setMinimumHeight(350), i.setMinimumWidth(350)
         i.selectAll(), i.setSpacing(5)
         self.delegate = ImageItemDelegate(self)
@@ -228,8 +248,8 @@ class CompressImagesProgress(Dialog):
         self.keep_going = True
         self.result = (None, '')
         Dialog.__init__(self, _('Compressing images...'), 'compress-images-progress', parent=parent)
-        self.gui_loop.connect(self.update_progress, type=Qt.QueuedConnection)
-        self.cidone.connect(self.accept, type=Qt.QueuedConnection)
+        self.gui_loop.connect(self.update_progress, type=Qt.ConnectionType.QueuedConnection)
+        self.cidone.connect(self.accept, type=Qt.ConnectionType.QueuedConnection)
         t = Thread(name='RunCompressImages', target=self.run_compress)
         t.daemon = True
         t.start()
@@ -250,25 +270,25 @@ class CompressImagesProgress(Dialog):
 
     def setup_ui(self):
         self.setWindowIcon(QIcon(I('compress-image.png')))
-        self.setCursor(Qt.BusyCursor)
+        self.setCursor(Qt.CursorShape.BusyCursor)
         self.setMinimumWidth(350)
         self.l = l = QVBoxLayout(self)
         self.la = la = QLabel(_('Compressing images, please wait...'))
-        la.setStyleSheet('QLabel { font-weight: bold }'), la.setAlignment(Qt.AlignCenter), la.setTextFormat(Qt.PlainText)
+        la.setStyleSheet('QLabel { font-weight: bold }'), la.setAlignment(Qt.AlignmentFlag.AlignCenter), la.setTextFormat(Qt.TextFormat.PlainText)
         l.addWidget(la)
         self.progress = p = QProgressBar(self)
         p.setMinimum(0), p.setMaximum(0)
         l.addWidget(p)
         self.msg = la = QLabel('\xa0')
-        la.setAlignment(Qt.AlignCenter), la.setTextFormat(Qt.PlainText)
+        la.setAlignment(Qt.AlignmentFlag.AlignCenter), la.setTextFormat(Qt.TextFormat.PlainText)
         l.addWidget(la)
 
-        self.bb.setStandardButtons(self.bb.Cancel)
+        self.bb.setStandardButtons(QDialogButtonBox.StandardButton.Cancel)
         l.addWidget(self.bb)
 
     def reject(self):
         self.keep_going = False
-        self.bb.button(self.bb.Cancel).setEnabled(False)
+        self.bb.button(QDialogButtonBox.StandardButton.Cancel).setEnabled(False)
         Dialog.reject(self)
 
     def progress_callback(self, num, total, name):
@@ -290,7 +310,7 @@ if __name__ == '__main__':
     c = get_container(sys.argv[-1], tweak_mode=True)
     set_current_container(c)
     d = CompressImages()
-    if d.exec_() == d.Accepted:
+    if d.exec_() == QDialog.DialogCode.Accepted:
         pass
     sip.delete(app)
     del app
