@@ -7,15 +7,15 @@ import os
 import shutil
 import sys
 from itertools import count
-from PyQt5.Qt import (
+from qt.core import (
     QT_VERSION, QApplication, QBuffer, QByteArray, QEvent, QFontDatabase, QFontInfo,
     QHBoxLayout, QIODevice, QLocale, QMimeData, QPalette, QSize, Qt, QTimer, QUrl,
-    QWidget, pyqtSignal
+    QWidget, pyqtSignal, sip
 )
-from PyQt5.QtWebEngineCore import (
+from qt.webengine import (
     QWebEngineUrlRequestInfo, QWebEngineUrlRequestJob, QWebEngineUrlSchemeHandler
 )
-from PyQt5.QtWebEngineWidgets import (
+from qt.webengine import (
     QWebEnginePage, QWebEngineProfile, QWebEngineScript, QWebEngineSettings,
     QWebEngineView
 )
@@ -28,7 +28,7 @@ from calibre.constants import (
 from calibre.ebooks.metadata.book.base import field_metadata
 from calibre.ebooks.oeb.polish.utils import guess_type
 from calibre.gui2 import choose_images, error_dialog, safe_open_url
-from calibre.gui2.viewer import link_prefix_for_location_links
+from calibre.gui2.viewer import link_prefix_for_location_links, performance_monitor
 from calibre.gui2.viewer.config import viewer_config_dir, vprefs
 from calibre.gui2.viewer.tts import TTS
 from calibre.gui2.webengine import (
@@ -38,14 +38,9 @@ from calibre.gui2.webengine import (
 from calibre.srv.code import get_translations_data
 from calibre.utils.serialize import json_loads
 from calibre.utils.shared_file import share_open
-from polyglot.builtins import as_bytes, iteritems, unicode_type
+from polyglot.builtins import as_bytes, iteritems
 from polyglot.functools import lru_cache
 
-try:
-    from PyQt5 import sip
-except ImportError:
-    import sip
-
 SANDBOX_HOST = FAKE_HOST.rpartition('.')[0] + '.sandbox'
 
 # Override network access to load data from the book {{{
@@ -80,7 +75,7 @@ def get_data(name):
     try:
         with share_open(path, 'rb') as f:
             return f.read(), guess_type(name)
-    except EnvironmentError as err:
+    except OSError as err:
         prints('Failed to read from book file: {} with error: {}'.format(name, as_unicode(err)))
     return None, None
 
@@ -127,7 +122,7 @@ def handle_mathjax_request(rq, name):
         try:
             with lopen(path, 'rb') as f:
                 raw = f.read()
-        except EnvironmentError as err:
+        except OSError as err:
             prints("Failed to get mathjax file: {} with error: {}".format(name, err), file=sys.stderr)
             rq.fail(QWebEngineUrlRequestJob.Error.RequestFailed)
             return
@@ -248,6 +243,7 @@ class ViewerBridge(Bridge):
     toggle_lookup = from_js(object)
     show_search = from_js(object, object)
     search_result_not_found = from_js(object)
+    search_result_discovered = from_js(object)
     find_next = from_js(object)
     quit = from_js()
     update_current_toc_nodes = from_js(object)
@@ -277,6 +273,8 @@ class ViewerBridge(Bridge):
     open_url = from_js(object)
     speak_simple_text = from_js(object)
     tts = from_js(object, object)
+    edit_book = from_js(object, object, object)
+    show_book_folder = from_js()
 
     create_view = to_js()
     start_book_load = to_js()
@@ -293,6 +291,7 @@ class ViewerBridge(Bridge):
     generic_action = to_js()
     show_search_result = to_js()
     prepare_for_close = to_js()
+    repair_after_fullscreen_switch = to_js()
     viewer_font_size_changed = to_js()
     tts_event = to_js()
 
@@ -367,7 +366,7 @@ class WebPage(QWebEnginePage):
         prints('%s: %s:%s: %s' % (prefix, source_id, linenumber, msg), file=sys.stderr)
         try:
             sys.stderr.flush()
-        except EnvironmentError:
+        except OSError:
             pass
 
     def acceptNavigationRequest(self, url, req_type, is_main_frame):
@@ -449,6 +448,7 @@ class WebView(RestartingWebEngineView):
     toggle_toc = pyqtSignal()
     show_search = pyqtSignal(object, object)
     search_result_not_found = pyqtSignal(object)
+    search_result_discovered = pyqtSignal(object)
     find_next = pyqtSignal(object)
     toggle_bookmarks = pyqtSignal()
     toggle_highlights = pyqtSignal()
@@ -475,6 +475,7 @@ class WebView(RestartingWebEngineView):
     scrollbar_context_menu = pyqtSignal(object, object, object)
     close_prep_finished = pyqtSignal(object)
     highlights_changed = pyqtSignal(object)
+    edit_book = pyqtSignal(object, object, object)
     shortcuts_changed = pyqtSignal(object)
     paged_mode_changed = pyqtSignal()
     standalone_misc_settings_changed = pyqtSignal(object)
@@ -496,6 +497,7 @@ class WebView(RestartingWebEngineView):
         self.show_home_page_on_ready = True
         self._size_hint = QSize(int(w/3), int(w/2))
         self._page = WebPage(self)
+        self._page.linkHovered.connect(self.link_hovered)
         self.view_is_ready = False
         self.bridge.bridge_ready.connect(self.on_bridge_ready)
         self.bridge.view_created.connect(self.on_view_created)
@@ -506,6 +508,7 @@ class WebView(RestartingWebEngineView):
         self.bridge.toggle_toc.connect(self.toggle_toc)
         self.bridge.show_search.connect(self.show_search)
         self.bridge.search_result_not_found.connect(self.search_result_not_found)
+        self.bridge.search_result_discovered.connect(self.search_result_discovered)
         self.bridge.find_next.connect(self.find_next)
         self.bridge.toggle_bookmarks.connect(self.toggle_bookmarks)
         self.bridge.toggle_highlights.connect(self.toggle_highlights)
@@ -533,6 +536,8 @@ class WebView(RestartingWebEngineView):
         self.bridge.scrollbar_context_menu.connect(self.scrollbar_context_menu)
         self.bridge.close_prep_finished.connect(self.close_prep_finished)
         self.bridge.highlights_changed.connect(self.highlights_changed)
+        self.bridge.edit_book.connect(self.edit_book)
+        self.bridge.show_book_folder.connect(self.show_book_folder)
         self.bridge.open_url.connect(safe_open_url)
         self.bridge.speak_simple_text.connect(self.tts.speak_simple_text)
         self.bridge.tts.connect(self.tts.action)
@@ -549,6 +554,11 @@ class WebView(RestartingWebEngineView):
             self.inspector = Inspector(parent.inspector_dock.toggleViewAction(), self)
             parent.inspector_dock.setWidget(self.inspector)
 
+    def link_hovered(self, url):
+        if url == 'javascript:void(0)':
+            url = ''
+        self.generic_action('show-status-message', {'text': url})
+
     def shutdown(self):
         self.tts.shutdown()
 
@@ -614,6 +624,7 @@ class WebView(RestartingWebEngineView):
         }
         self.bridge.create_view(
             vprefs['session_data'], vprefs['local_storage'], field_metadata.all_metadata(), ui_data)
+        performance_monitor('bridge ready')
         for func, args in iteritems(self.pending_bridge_ready_actions):
             getattr(self.bridge, func)(*args)
 
@@ -638,8 +649,8 @@ class WebView(RestartingWebEngineView):
     def goto_toc_node(self, node_id):
         self.execute_when_ready('goto_toc_node', node_id)
 
-    def goto_cfi(self, cfi):
-        self.execute_when_ready('goto_cfi', cfi)
+    def goto_cfi(self, cfi, add_to_history=False):
+        self.execute_when_ready('goto_cfi', cfi, bool(add_to_history))
 
     def notify_full_screen_state_change(self, in_fullscreen_mode):
         self.execute_when_ready('full_screen_state_changed', in_fullscreen_mode)
@@ -670,7 +681,7 @@ class WebView(RestartingWebEngineView):
             vprefs['local_storage'] = sd
 
     def do_callback(self, func_name, callback):
-        cid = unicode_type(next(self.callback_id_counter))
+        cid = str(next(self.callback_id_counter))
         self.callback_map[cid] = callback
         self.execute_when_ready('get_current_cfi', cid)
 
@@ -728,3 +739,10 @@ class WebView(RestartingWebEngineView):
 
     def tts_settings_changed(self, ui_settings):
         self.execute_when_ready('tts_event', 'configured', ui_settings)
+
+    def show_book_folder(self):
+        path = os.path.dirname(os.path.abspath(set_book_path.pathtoebook))
+        safe_open_url(QUrl.fromLocalFile(path))
+
+    def repair_after_fullscreen_switch(self):
+        self.execute_when_ready('repair_after_fullscreen_switch')
