@@ -1,4 +1,3 @@
-
 '''
 Created on 23 Sep 2010
 
@@ -15,12 +14,13 @@ from math import modf
 
 from calibre import prints
 from calibre.constants import DEBUG
+from calibre.ebooks.metadata.book.base import field_metadata
 from calibre.utils.formatter_functions import formatter_functions
 from calibre.utils.icu import strcmp
-from polyglot.builtins import unicode_type, error_message
+from polyglot.builtins import error_message
 
 
-class Node(object):
+class Node:
     NODE_RVALUE = 1
     NODE_IF = 2
     NODE_ASSIGN = 3
@@ -255,7 +255,7 @@ class CharacterNode(Node):
         self.expression = expression
 
 
-class _Parser(object):
+class _Parser:
     LEX_OP = 1
     LEX_ID = 2
     LEX_CONST = 3
@@ -539,8 +539,8 @@ class _Parser(object):
                              lambda ln, args: IfNode(ln, args[0], (args[1],), (args[2],))),
         'first_non_empty':  (lambda args: len(args) == 1,
                              lambda ln, args: FirstNonEmptyNode(ln, args)),
-        'assign':           (lambda args: len(args) == 2 and args[0].node_type == Node.NODE_RVALUE,
-                             lambda ln, args: AssignNode(ln, args[0].name, args[1])),
+        'assign':           (lambda args: len(args) == 2 and len(args[0]) == 1 and args[0][0].node_type == Node.NODE_RVALUE,
+                             lambda ln, args: AssignNode(ln, args[0][0].name, args[1])),
         'contains':         (lambda args: len(args) == 4,
                              lambda ln, args: ContainsNode(ln, args)),
         'character':        (lambda args: len(args) == 1,
@@ -677,7 +677,7 @@ class StopException(Exception):
         super().__init__('Template evaluation stopped')
 
 
-class _Interpreter(object):
+class _Interpreter:
     def error(self, message, line_number):
         m = _('Interpreter: {0} - line number {1}').format(message, line_number)
         raise ValueError(m)
@@ -890,6 +890,7 @@ class _Interpreter(object):
     def do_node_raw_field(self, prog):
         try:
             name = self.expr(prog.expression)
+            name = field_metadata.search_term_to_field_key(name)
             res = getattr(self.parent_book, name, None)
             if res is None and prog.default is not None:
                 res = self.expr(prog.default)
@@ -904,9 +905,9 @@ class _Interpreter(object):
                     else:
                         res = fm['is_multiple']['list_to_ui'].join(res)
                 else:
-                    res = unicode_type(res)
+                    res = str(res)
             else:
-                res = unicode_type(res)  # Should be the string "None"
+                res = str(res)  # Should be the string "None"
             if (self.break_reporter):
                 self.break_reporter(prog.node_name, res, prog.line_number)
             return res
@@ -925,8 +926,9 @@ class _Interpreter(object):
 
     def do_node_first_non_empty(self, prog):
         for expr in prog.expression_list:
-            if v := self.expr(expr):
-                if (self.break_reporter):
+            v = self.expr(expr)
+            if v:
+                if self.break_reporter:
                     self.break_reporter(prog.node_name, v, prog.line_number)
                 return v
         if (self.break_reporter):
@@ -1041,7 +1043,7 @@ class _Interpreter(object):
         try:
             answer = self.ARITHMETIC_BINARY_OPS[prog.operator](float(self.expr(prog.left)),
                                                                float(self.expr(prog.right)))
-            res = unicode_type(answer if modf(answer)[0] != 0 else int(answer))
+            res = str(answer if modf(answer)[0] != 0 else int(answer))
             if (self.break_reporter):
                 self.break_reporter(prog.node_name, res, prog.line_number)
             return res
@@ -1059,7 +1061,7 @@ class _Interpreter(object):
     def do_node_unary_arithop(self, prog):
         try:
             expr = self.ARITHMETIC_UNARY_OPS[prog.operator](float(self.expr(prog.expr)))
-            res = unicode_type(expr if modf(expr)[0] != 0 else int(expr))
+            res = str(expr if modf(expr)[0] != 0 else int(expr))
             if (self.break_reporter):
                 self.break_reporter(prog.node_name, res, prog.line_number)
             return res
@@ -1154,10 +1156,15 @@ class TemplateFormatter(string.Formatter):
         self.book = None
         self.kwargs = None
         self.strip_results = True
+        self.column_name = None
+        self.template_cache = None
+        self.global_vars = {}
         self.locals = {}
         self.funcs = formatter_functions().get_functions()
-        self.gpm_parser = _Parser()
-        self.gpm_interpreter = _Interpreter()
+        self._interpreters = []
+        self._template_parser = None
+        self.recursion_stack = []
+        self.recursion_level = -1
 
     def _do_format(self, val, fmt):
         if not fmt or not val:
@@ -1179,7 +1186,7 @@ class TemplateFormatter(string.Formatter):
             except:
                 raise ValueError(
                     _('format: type {0} requires a decimal (float) value, got {1}').format(typ, val))
-        return unicode_type(('{0:'+fmt+'}').format(val))
+        return str(('{0:'+fmt+'}').format(val))
 
     def _explode_format_string(self, fmt):
         try:
@@ -1253,7 +1260,7 @@ class TemplateFormatter(string.Formatter):
         # ensure we are dealing with a string.
         if isinstance(val, numbers.Number):
             if val:
-                val = unicode_type(val)
+                val = str(val)
             else:
                 val = ''
         # Handle conditional text
@@ -1333,17 +1340,77 @@ class TemplateFormatter(string.Formatter):
             ans = ans.strip(' ')
         return ans
 
+    # It is possible for a template to indirectly invoke other templates by
+    # doing field references of composite columns. If this happens then the
+    # reference can use different parameters when calling safe_format(). Because
+    # the parameters are saved as instance variables they can possibly affect
+    # the 'calling' template. To avoid this problem, save the current formatter
+    # state when recursion is detected. There is no point in saving the level
+    # 0 state.
+
+    def save_state(self):
+        self.recursion_level += 1
+        if self.recursion_level > 0:
+            return (
+                (self.strip_results,
+                 self.column_name,
+                 self.template_cache,
+                 self.kwargs,
+                 self.book,
+                 self.global_vars,
+                 self.funcs,
+                 self.locals))
+        else:
+            return None
+
+    def restore_state(self, state):
+        self.recursion_level -= 1
+        if state is not None:
+            (self.strip_results,
+             self.column_name,
+             self.template_cache,
+             self.kwargs,
+             self.book,
+             self.global_vars,
+             self.funcs,
+             self.locals) = state
+
+    # Allocate an interpreter if the formatter encounters a GPM or TPM template.
+    # We need to allocate additional interpreters if there is composite recursion
+    # so that the templates are evaluated by separate instances. It is OK to
+    # reuse already-allocated interpreters because their state is initialized on
+    # call. As a side effect, no interpreter is instantiated if no TPM/GPM
+    # template is encountered.
+
+    @property
+    def gpm_interpreter(self):
+        while len(self._interpreters) <= self.recursion_level:
+            self._interpreters.append(_Interpreter())
+        return self._interpreters[self.recursion_level]
+
+    # Allocate a parser if needed. Parsers cannot recurse so one is sufficient.
+
+    @property
+    def gpm_parser(self):
+        if self._template_parser is None:
+            self._template_parser = _Parser()
+        return self._template_parser
+
     # ######### a formatter that throws exceptions ############
 
     def unsafe_format(self, fmt, kwargs, book, strip_results=True, global_vars=None):
-        self.strip_results = strip_results
-        self.column_name = self.template_cache = None
-        self.kwargs = kwargs
-        self.book = book
-        self.composite_values = {}
-        self.locals = {}
-        self.global_vars = global_vars if isinstance(global_vars, dict) else {}
-        return self.evaluate(fmt, [], kwargs, self.global_vars)
+        state = self.save_state()
+        try:
+            self.strip_results = strip_results
+            self.column_name = self.template_cache = None
+            self.kwargs = kwargs
+            self.book = book
+            self.composite_values = {}
+            self.locals = {}
+            self.global_vars = global_vars if isinstance(global_vars, dict) else {}
+            return self.evaluate(fmt, [], kwargs, self.global_vars)
+        finally:
+            self.restore_state(state)
 
     # ######### a formatter guaranteed not to throw an exception ############
 
@@ -1351,30 +1418,36 @@ class TemplateFormatter(string.Formatter):
                     column_name=None, template_cache=None,
                     strip_results=True, template_functions=None,
                     global_vars=None, break_reporter=None):
-        self.strip_results = strip_results
-        self.column_name = column_name
-        self.template_cache = template_cache
-        self.kwargs = kwargs
-        self.book = book
-        self.global_vars = global_vars if isinstance(global_vars, dict) else {}
-        if template_functions:
-            self.funcs = template_functions
-        else:
-            self.funcs = formatter_functions().get_functions()
-        self.composite_values = {}
-        self.locals = {}
+        state = self.save_state()
+        if self.recursion_level == 0:
+            # Initialize the composite values dict if this is the base-level
+            # call. Recursive calls will use the same dict.
+            self.composite_values = {}
         try:
-            ans = self.evaluate(fmt, [], kwargs, self.global_vars,
-                                break_reporter=break_reporter)
-        except StopException as e:
-            ans = error_message(e)
-        except Exception as e:
-            if DEBUG:  # and getattr(e, 'is_locking_error', False):
-                traceback.print_exc()
-                if column_name:
-                    prints('Error evaluating column named:', column_name)
-            ans = error_value + ' ' + error_message(e)
-        return ans
+            self.strip_results = strip_results
+            self.column_name = column_name
+            self.template_cache = template_cache
+            self.kwargs = kwargs
+            self.book = book
+            self.global_vars = global_vars if isinstance(global_vars, dict) else {}
+            if template_functions:
+                self.funcs = template_functions
+            else:
+                self.funcs = formatter_functions().get_functions()
+            self.locals = {}
+            try:
+                ans = self.evaluate(fmt, [], kwargs, self.global_vars, break_reporter=break_reporter)
+            except StopException as e:
+                ans = error_message(e)
+            except Exception as e:
+                if DEBUG:  # and getattr(e, 'is_locking_error', False):
+                    traceback.print_exc()
+                    if column_name:
+                        prints('Error evaluating column named:', column_name)
+                ans = error_value + ' ' + error_message(e)
+            return ans
+        finally:
+            self.restore_state(state)
 
 
 class ValidateFormatter(TemplateFormatter):
